﻿// $Id: ButtonRenderer.cs 73 2012-04-11 01:37:42Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2009 - 2012
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;

namespace Twp.Controls
{
    public static class ButtonRenderer
    {
        public static void DrawButton( Graphics g, Rectangle rect, PushButtonState state )
        {
            if( Application.RenderWithVisualStyles )
            {
                System.Windows.Forms.ButtonRenderer.DrawButton( g, rect, state );
            }
            else
            {
                ButtonState bState = ButtonState.Normal;
                switch( state )
                {
                    case PushButtonState.Disabled:
                        bState = ButtonState.Inactive;
                        break;
                    case PushButtonState.Pressed:
                        bState = ButtonState.Pushed;
                        break;
                }
                ControlPaint.DrawButton( g, rect, bState );
            }
        }

        public static void DrawArrow( Graphics g, Rectangle rect, Color color, ArrowDirection direction )
        {
            Point center = new Point( rect.X + (rect.Width / 2), rect.Y + (rect.Height / 2) );
            Point[] arrowPoints = new Point[3];
            Point[] linePoints = new Point[2];
            switch( direction )
            {
                case ArrowDirection.Up:
                    arrowPoints[0] = new Point( center.X, center.Y - 3 );
                    arrowPoints[1] = new Point( center.X + 5, center.Y + 2 );
                    arrowPoints[2] = new Point( center.X - 4, center.Y + 2 );
                    linePoints[0] = new Point( center.X - 3, center.Y - 3 );
                    linePoints[1] = new Point( center.X + 3, center.Y - 3 );
                    break;
                case ArrowDirection.Down:
                    arrowPoints[0] = new Point( center.X - 3, center.Y - 2 );
                    arrowPoints[1] = new Point( center.X + 4, center.Y - 2 );
                    arrowPoints[2] = new Point( center.X, center.Y + 2 );
                    linePoints[0] = new Point( center.X - 3, center.Y + 2 );
                    linePoints[1] = new Point( center.X + 3, center.Y + 2 );
                    break;
                case ArrowDirection.Left:
                    arrowPoints[0] = new Point( center.X + 2, center.Y - 4 );
                    arrowPoints[1] = new Point( center.X + 2, center.Y + 4 );
                    arrowPoints[2] = new Point( center.X - 2, center.Y );
                    linePoints[0] = new Point( center.X - 3, center.Y - 3 );
                    linePoints[1] = new Point( center.X - 3, center.Y + 3 );
                    break;
                case ArrowDirection.Right:
                    arrowPoints[0] = new Point( center.X - 2, center.Y - 4 );
                    arrowPoints[1] = new Point( center.X - 2, center.Y + 4 );
                    arrowPoints[2] = new Point( center.X + 2, center.Y );
                    linePoints[0] = new Point( center.X + 2, center.Y - 3 );
                    linePoints[1] = new Point( center.X + 2, center.Y + 3 );
                    break;
            }
            using( Brush brush = new SolidBrush( color ) )
            {
                g.FillPolygon( brush, arrowPoints );
            }
            using( Pen pen = new Pen( color ) )
            {
                g.DrawLine( pen, linePoints[0], linePoints[1] );
            }
        }
    }
}
