﻿// $Id: FlagEditor.cs 83 2012-10-15 18:45:03Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2009 - 2011
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System;
using System.ComponentModel;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using Twp.Utilities;

namespace Twp.Controls.Design
{
    public class FlagsEditor : UITypeEditor
    {
        private FlagsCheckedListBox checkBox;

        public FlagsEditor()
        {
            this.checkBox = new FlagsCheckedListBox();
            this.checkBox.BorderStyle = BorderStyle.None;
        }

        public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, object value )
        {
            if( context != null && context.Instance != null && provider != null )
            {
                IWindowsFormsEditorService editorService = (IWindowsFormsEditorService) provider.GetService( typeof( IWindowsFormsEditorService ) );
                if( editorService != null )
                {
                    Enum e = (Enum) Convert.ChangeType( value, context.PropertyDescriptor.PropertyType );
                    this.checkBox.EnumValue = e;
                    editorService.DropDownControl( this.checkBox );
                    return this.checkBox.EnumValue;
                }
            }
            return null;
        }

        public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
        {
            return UITypeEditorEditStyle.DropDown;
        }
    }

    public class FlagsCheckedListBox : CheckedListBox
    {
        Container components = null;

        public FlagsCheckedListBox()
        {
            // This call is required by the Windows.Forms Form Designer.
            InitializeComponent();

            // TODO: Add any initialization after the InitForm call
        }

        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if( this.components != null )
                    this.components.Dispose();
            }
            base.Dispose( disposing );
        }

        #region Component Designer generated code
        private void InitializeComponent()
        {
            ///
            /// FlagsCkechedListBox
            /// 
            this.CheckOnClick = true;
        }
        #endregion

        public FlagsCheckedListBoxItem Add( int v, string c )
        {
            FlagsCheckedListBoxItem item = new FlagsCheckedListBoxItem( v, c );
            this.Items.Add( item );
            return item;
        }

        public FlagsCheckedListBoxItem Add( FlagsCheckedListBoxItem item )
        {
            this.Items.Add( item );
            return item;
        }

        protected override void OnItemCheck( ItemCheckEventArgs e )
        {
            base.OnItemCheck( e );

            if( this.isUpdatingCheckStates )
                return;

            FlagsCheckedListBoxItem item = Items[e.Index] as FlagsCheckedListBoxItem;
            this.UpdateCheckedItems( item, e.NewValue );
        }

        protected void UpdateCheckedItems( int value )
        {
            this.isUpdatingCheckStates = true;

            for( int i = 0; i < this.Items.Count; i++ )
            {
                FlagsCheckedListBoxItem item = this.Items[i] as FlagsCheckedListBoxItem;
                if( item.Value == 0 )
                {
                    this.SetItemChecked( i, value == 0 );
                }
                else
                {
                    if( (item.Value & value) == item.Value )
                        this.SetItemChecked( i, true );
                    else
                        this.SetItemChecked( i, false );
                }
            }

            this.isUpdatingCheckStates = false;
        }

        protected void UpdateCheckedItems( FlagsCheckedListBoxItem composite, CheckState state )
        {
            if( composite.Value == 0 )
                UpdateCheckedItems( 0 );

            int sum = this.GetCurrentValue();

            if( state == CheckState.Unchecked )
                sum = sum & (~composite.Value);
            else
                sum |= composite.Value;

            UpdateCheckedItems( sum );
        }

        private bool isUpdatingCheckStates = false;

        public int GetCurrentValue()
        {
            int sum = 0;
            for( int i = 0; i < this.Items.Count; i++ )
            {
                if( this.GetItemChecked( i ) )
                {
                    FlagsCheckedListBoxItem item = this.Items[i] as FlagsCheckedListBoxItem;
                    sum |= item.Value;
                }
            }
            return sum;
        }

        Type enumType;
        Enum enumValue;

        private void FillEnumMembers()
        {
            foreach( string name in Enum.GetNames( this.enumType ) )
            {
                object val = Enum.Parse( this.enumType, name );
                int intVal = (int) Convert.ChangeType( val, typeof( int ) );
                this.Add( intVal, name );
            }
        }

        private void ApplyEnumValue()
        {
            int intVal = (int) Convert.ChangeType( this.enumValue, typeof( int ) );
            this.UpdateCheckedItems( intVal );
        }

        [DesignerSerializationVisibility( DesignerSerializationVisibility.Hidden )]
        public Enum EnumValue
        {
            get
            {
                object e = Enum.ToObject( this.enumType, this.GetCurrentValue() );
                return (Enum) e;
            }
            set
            {
                if( this.enumValue != value )
                {
                    this.Items.Clear();
                    this.enumValue = value;
                    this.enumType = value.GetType();
                    this.FillEnumMembers();
                    this.ApplyEnumValue();
                }
            }
        }
    }

    public class FlagsCheckedListBoxItem
    {
        public FlagsCheckedListBoxItem( int value, string name )
        {
            this.value = value;
            this.name = name;
        }

        private int value;
        public int Value
        {
            get { return this.value; }
            set { this.value = value; }
        }

        private string name;
        public string Name
        {
            get { return this.name; }
            set { this.name = value; }
        }

        public bool IsFlag
        {
            get
            {
                return ((this.value & (this.value - 1)) == 0);
            }
        }

        public bool IsMemberFlag( FlagsCheckedListBoxItem composite )
        {
            return (this.IsFlag && ((this.value & composite.value) == value));
        }

        public override string ToString()
        {
            return this.name;
        }
    }
}
