﻿// $Id: ProgressForm.cs 63 2012-02-29 17:36:59Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2009 - 2012
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System;
using System.Windows.Forms;

namespace Twp.Controls
{
	public partial class ProgressForm : Form
	{
		public ProgressForm()
		{
			InitializeComponent();
		}

		public int Total
		{
			get { return this.overallProgressBar.Maximum; }
			set { this.overallProgressBar.Maximum = value; }
		}

		public int Maximum
		{
			get { return this.progressBar.Maximum; }
			set
			{
				this.progressBar.Maximum = value;
				this.progressBar.Value = 0;
			}
		}

		public void PerformStep( int value, string text )
		{
			this.progressLabel.Text = text;
			this.progressBar.Value += value;
			this.overallProgressBar.Value += value;
			int overall = (this.overallProgressBar.Value * 100) / this.overallProgressBar.Maximum;
			this.overallProgressLabel.Text = String.Format( "Overall progress: {0}%", overall );
		}

		public string Label
		{
			get { return this.label.Text; }
			set { this.label.Text = value; }
		}

		private int autoCloseDelay = 0;
		public int AutoCloseDelay
		{
			get { return this.autoCloseDelay; }
			set { this.autoCloseDelay = value; }
		}

		public bool CanBeCancelled
		{
			get { return this.cancelButton.Enabled; }
			set { this.cancelButton.Enabled = value; }
		}

		private bool canceled = false;
		public bool Canceled
		{
			get { return this.canceled; }
		}

		public event EventHandler Cancel;

		private void OnCancel( object sender, EventArgs e )
		{
			this.canceled = true;

			if( this.Cancel != null )
				this.Cancel( this, EventArgs.Empty );
		}

		private void OnFormClosing( object sender, FormClosingEventArgs e )
		{
			if( this.autoCloseDelay > 0 )
			{
				this.cancelButton.Visible = false;
				this.closeButton.Visible = true;
				this.closeButton.Text = String.Format( "Close ({0})", this.autoCloseDelay );
				this.timer.Start();
				e.Cancel = true;
			}
		}

		private void OnTimerTick( object sender, EventArgs e )
		{
			this.autoCloseDelay--;
			this.closeButton.Text = String.Format( "Close ({0})", this.autoCloseDelay );
			if( this.autoCloseDelay < 0 )
			{
				this.OnClose( this, EventArgs.Empty );
			}
		}

		private void OnClose( object sender, EventArgs e )
		{
			this.timer.Stop();
			this.autoCloseDelay = 0;
			this.Close();
		}
	}
}
