﻿// $Id: VerticalLayout.cs 72 2012-04-10 20:58:06Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2005 - 2012
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.Layout;

namespace Twp.Controls
{
    public class VerticalLayout : LayoutEngine
    {
        #region Properties

        /// <summary>
        /// Returns a value equal to the full height used by the controls in the container.
        /// </summary>
        public int Height
        {
            get { return this.height; }
        }
        private int height = 0;

        /// <summary>
        /// Gets or sets a value representing the height of the space between controls.
        /// </summary>
        /// <remarks>
        /// If set to -1, the layout will use each control's bottom margin for spacing. This is the default.
        /// </remarks>
        public int Spacing
        {
            get { return this.spacing; }
            set { this.spacing = value; }
        }
        private int spacing = -1;

        /// <summary>
        /// Gets or sets a value indicating whether or not hidden controls are included by the LayoutEngine.
        /// </summary>
        public bool IncludeHidden
        {
            get { return this.includeHidden; }
            set { this.includeHidden = value; }
        }
        private bool includeHidden = false;

        #endregion

        #region Methods

        public override bool Layout( object container, System.Windows.Forms.LayoutEventArgs layoutEventArgs )
        {
            Control parent = container as Control;

            Rectangle pDisplayRect = parent.DisplayRectangle;
            Point nextLocation = pDisplayRect.Location;
            nextLocation.Offset( parent.Margin.Left, parent.Margin.Top );

            foreach( Control control in parent.Controls )
            {
                if( !control.Visible && !this.includeHidden )
                    continue;

                control.Location = nextLocation;
                control.Width = pDisplayRect.Width - parent.Margin.Horizontal;

                if( this.spacing >= 0 )
                    nextLocation.Offset( 0, control.Height + this.spacing );
                else
                    nextLocation.Offset( 0, control.Height + control.Margin.Bottom );
            }

            this.height = nextLocation.Y + parent.Margin.Bottom;
            return true;
        }

        #endregion
    }
}
