﻿// $Id: ArchiveItem.cs 7 2011-05-20 15:56:53Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2009 - 2011
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System;
using System.Collections.Generic;
using System.Xml;

namespace Twp.AO.Xml
{
	public class ArchiveItem : IAOElement
	{
		public ArchiveItem()
		{
			this.type = "Unknown";
			this.name = String.Empty;
			this.value = String.Empty;
		}

		public ArchiveItem( string type, string value )
		{
			this.type = type;
			this.name = String.Empty;
			this.value = value;
		}

		public ArchiveItem( string type, string name, string value )
		{
			this.type = type;
			this.name = name;
			this.value = value;
		}

		private string type;
		public string Type
		{
			get { return this.type; }
			set { this.type = value; }
		}

		private string name;
		public string Name
		{
			get { return this.name; }
			set { this.name = value; }
		}

		private string value;
		public string Value
		{
			get { return this.value; }
			set { this.value = value; }
		}

		public bool HasChildren
		{
			get { return false; }
		}

		public void ReadXml( XmlReader reader )
		{
			this.type = reader.Name;
			if( !reader.HasAttributes )
				return;

			this.name = reader.GetAttribute( "name" );
			this.value = reader.GetAttribute( "value" );
		}

		public void WriteXml( XmlWriter writer )
		{
			writer.WriteStartElement( this.type );
			if( !String.IsNullOrEmpty( this.name ) )
			{
				writer.WriteAttributeString( "name", this.name );
			}
			writer.WriteAttributeString( "value", this.value );
			writer.WriteEndElement();
		}

		public IAOElement GetElement( string name )
		{
			if( this.name == name )
				return this;
			return null;
		}

		public override string ToString()
		{
			if( String.IsNullOrEmpty( this.name ) )
				return String.Format( "[{0}] {1}", this.type, this.value );
			else
				return String.Format( "[{0}] {1}={2}", this.type, this.name, this.value );
		}
	}
}
